module SBTree where

data SBTree = L | N SBTree SBTree 
                         deriving (Show, Eq)

type Depth = Integer

genSBTree :: Depth -> SBTree
genSBTree 0 = L
genSBTree (n+1) = N (genSBTree n) (genSBTree n)

nodes :: SBTree -> Integer
nodes L = 1
nodes (N leftT rightT) = 1 + nodes leftT + nodes rightT

joinTrees :: SBTree -> SBTree -> SBTree
joinTrees leftTree rightTree = N leftTree rightTree

depth :: SBTree -> Integer
depth L = 0
depth (N lt rt) = (max (depth lt) (depth rt)) + 1

balanced :: SBTree -> Bool
balanced L = True
balanced (N lt rt) = (balanced lt) && (balanced rt) && depth lt == depth rt


{-- Zeichenfunktion fr die SBTrees
    Author: Carl Witt 
    ProInformatik II 2010
--}

printSBTree tree = printStringList (fst (toPic tree))

toPic L = (["(L) "],1)        -- lines + zero based root symbol position index

-- the root line consists of the symbol N, the rootBar is the "|" character below it
-- the vertical line represents the "---"-bar between the two subTrees and
-- the subBars line consists of two "|" characters over the underlying root positions

toPic (N l r) = ([rootLine, rootBar, vertical, subBars]++joinedTrees,newRootPos)
        where
            lt = toPic l; rt = toPic r                -- recursive result
            ltPic = fst lt; rtPic = fst rt            -- raw "pictures" of left and right
            ltPicH = harmonize ltPic rtPic; rtPicH = harmonize rtPic ltPic
            lWidth = length (head ltPic); rWidth = length (head rtPic); totalWidth = lWidth + rWidth            

            ltRootPos = snd lt; rtRootPos = snd rt    
            newRootPos = ltRootPos + div barLength 2
            barLength = (lWidth - ltRootPos) + rtRootPos
        
            joinedTrees = zipWith (++) ltPicH rtPicH
            rootLine = makeN " " newRootPos ++ "N" ++ makeN " " (totalWidth-newRootPos-1)
            rootBar  = makeN " " newRootPos ++ "|" ++ makeN " " (totalWidth-newRootPos-1)
            vertical = makeN " " ltRootPos ++ "/" ++ makeN "-" (barLength-1) ++ "\\" ++ 
                                                     makeN " " (totalWidth-barLength-ltRootPos-1)
            subBars  = leftBar ++ rightBar
            leftBar  = makeN " " ltRootPos ++ "|" ++ makeN " " (lWidth - ltRootPos - 1)
            rightBar = makeN " " rtRootPos ++ "|" ++ makeN " " (rWidth - rtRootPos - 1)
    
 -- appends the missing empty lines where necessary
harmonize :: [String] -> [String] -> [String]
harmonize a toB = if length a >= length toB 
                        then a
                        else a ++ (take missing (repeat filler))
                            where 
                                lenA = length a; lenB = length toB
                                missing = lenB - lenA
                                filler = makeN " " (length (head a))
    
-- creates n concatenated instances of a (makeN "-" 3 => "---")
makeN :: [a] -> Int -> [a]
makeN a n = take n (foldr (++) [] (repeat a))
    
-- puts a string list with new lines after each string to the screen
printStringList list = putStr (huddled list)
        where
            huddled list = foldl (++) [] (map (++"\n") list)

